#version 450

struct Particle {
    vec4 position;
    vec4 velocity;
    float killPlaneHeight;
	float pad[3];
};

layout(location = 0) in vec3 in_position; // Unused for particles (we use SSBO)

layout (binding = 0) uniform UniformBuffer {
	vec4 color;
	mat4 modelMatrix;
	mat4 modelMatrixForNormals;
	mat4 viewProjMatrix;
	vec4 cameraPosition;
	vec4 illumination; // ka, kd, ks, alpha
	ivec4 userInput;
	
	float time;
    float impactX;
    float impactY;
    float pad3;
} ub_data;

layout(binding = 5, std430) buffer ParticleBuffer {
    Particle particles[];
};

layout(location = 1) out float vNormHeight;

const float waterfallTop = 5.0;
const float waterfallBottom = -0.5;

void main() {
    vec4 worldPos = ub_data.modelMatrix * particles[gl_VertexIndex].position;
	gl_Position = ub_data.viewProjMatrix * worldPos;

	vNormHeight = clamp((particles[gl_VertexIndex].position.y - waterfallBottom) /
                        (waterfallTop - waterfallBottom), 0.0, 1.0);

    float distance = length(worldPos.xyz - ub_data.cameraPosition.xyz);
	float referenceDistance = 5.0;
    float scale = referenceDistance / distance;

	float basePointSize = mix(12.0, 6.0, 1.0 - vNormHeight);
    gl_PointSize = basePointSize * scale;
}