#version 460

layout (binding = 0) uniform UniformBuffer {
    vec4 color;
    mat4 modelMatrix;
    mat4 modelMatrixForNormals;
    mat4 viewProjMatrix;
    vec4 cameraPosition;
    vec4 illumination;
    ivec4 userInput;

    float time;
    float impactX;
    float impactY;
    float pad3;
} ub_data;

layout (binding = 1) uniform DirectionalLight {
    vec4 color;
    vec4 direction;
} dl_data;

layout (binding = 2) uniform PointLight {
    vec4 color;
    vec4 position;
    vec4 attenuation;
} pl_data;

layout (binding = 3) uniform sampler2D diffuse_texture;
layout (binding = 4) uniform sampler2D normal_map;


// === TESC Input/Output ===

// We have 3 control points per patch:
layout(vertices = 3) out;

struct VertexData
{
    vec3 posOS;      
    vec3 normalOS;   
    vec2 texcoords;
};

// The TESC input array:
layout (location = 0) in VertexData tcs_in[];

// The TESC output array:
layout (location = 0) out VertexData tcs_out[];

void main()
{
    tcs_out[gl_InvocationID].posOS     = tcs_in[gl_InvocationID].posOS;
    tcs_out[gl_InvocationID].normalOS  = tcs_in[gl_InvocationID].normalOS;
    tcs_out[gl_InvocationID].texcoords = tcs_in[gl_InvocationID].texcoords;

    vec3 centerOS = (tcs_in[0].posOS + tcs_in[1].posOS + tcs_in[2].posOS) / 3.0;
    vec4 centerWorld4 = ub_data.modelMatrix * vec4(centerOS, 1.0);
    vec3 centerWorld = centerWorld4.xyz;

    float distance = length(centerWorld - ub_data.cameraPosition.xyz);
    float maxTessLevel = float(ub_data.userInput[3]);
    float minTessLevel = maxTessLevel / 3.0;

    float clampedDistance = clamp(distance, 2.0, 20.0);
    float t = (clampedDistance - 2.0) / (20.0 - 2.0); // Maps 2-20 to 0-1

    float tessLevel = mix(maxTessLevel, minTessLevel, t);

    gl_TessLevelOuter[0] = tessLevel;
    gl_TessLevelOuter[1] = tessLevel;
    gl_TessLevelOuter[2] = tessLevel;
    gl_TessLevelInner[0] = tessLevel;
}